"Main application."


#
#   MoviePaca, movie database manager.
#   Copyright (C) 2010  Joakim Soderlund
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


import gtk
import os.path

import util
import search
import genre
import role
import person
import movie


class WinMain():
	PAGE_MOVIE = 0
	PAGE_PERSON = 1
	PAGE_GENRE = 2
	PAGE_ROLE = 3
	
	def __init__(self):
		self.builder = gtk.Builder()
		self.builder.add_from_file(os.path.join(util.DATA_PATH, "glade", "main.glade"))
		self.builder.connect_signals(self)
		
		self.window = self.builder.get_object("win_main")
		self.notebook = self.builder.get_object("notebook")

		self.page_map = {
			self.PAGE_MOVIE : {
				"id" : "movie_id",
				"search" : search.MOVIE,
				"editor" : movie.DiaMovie,
				"view" : self.builder.get_object("tw_movie"),
				"store" : self.builder.get_object("ls_movie"),
				"default" : ("SELECT movie_id, title, year, duration FROM Movie", None),
				"create" : "INSERT INTO Movie (title, year, duration) VALUES ('None', 0, 0)",
				"delete" : ["Movie", "MovieStaff", "MovieGenre"]
			},
			self.PAGE_PERSON : {
				"id" : "person_id",
				"search" : search.PERSON,
				"editor" : person.DiaPerson,
				"view" : self.builder.get_object("tw_person"),
				"store" : self.builder.get_object("ls_person"),
				"default" : ("SELECT person_id, name FROM Person", None),
				"create" : "INSERT INTO Person (name) VALUES ('None')",
				"delete" : ["Person", "MovieStaff"]
			},
			self.PAGE_GENRE : {
				"id" : "genre_id",
				"search" : search.GENRE,
				"editor" : genre.DiaGenre,
				"view" : self.builder.get_object("tw_genre"),
				"store" : self.builder.get_object("ls_genre"),
				"default" : ("SELECT genre_id, genre FROM Genre", None),
				"create" : "INSERT INTO Genre (genre) VALUES ('None')",
				"delete" : ["Genre", "MovieGenre"],
			},
			self.PAGE_ROLE : {
				"id" : "role_id",
				"search" : search.ROLE,
				"editor" : role.DiaRole,
				"view" : self.builder.get_object("tw_role"),
				"store" : self.builder.get_object("ls_role"),
				"default" : ("SELECT role_id, role FROM Role", None),
				"create" : "INSERT INTO Role (role) VALUES ('None')",
				"delete" : ["Role", "MovieStaff"],
			},
		}
		
		for value in self.page_map.values():
			value["selection"] = value["view"].get_selection()
			value["selection"].set_mode(gtk.SELECTION_MULTIPLE)

	def gtk_main_quit(self, *args):
		gtk.main_quit()

	def open_dialog_search(self, *args):
		page = self.notebook.get_current_page()
		map = self.page_map[page]
		ds = search.DialogSearch(
			map["search"],
			lambda query, data: self.query(page, query, data),
		)
		ds.dialog.show()

	def open_dialog_about(self, *args):
		print("Not yet implemented!")
		print(args)
			
	def item_add(self, *args):
		page = self.notebook.get_current_page()
		map = self.page_map[page]
		cursor = util.create_cursor()
		cursor.execute(map["create"])
		editor = map["editor"](cursor.lastrowid, self.refresh)		
		cursor.close()
		editor.dialog.show()

	def item_remove(self, *args):
		dialog = gtk.MessageDialog(
			self.window,
			gtk.DIALOG_MODAL,
			gtk.MESSAGE_QUESTION,
			gtk.BUTTONS_YES_NO,
			"Remove selected items?"
		)
		
		response = dialog.run()
		dialog.destroy()
		
		if response == gtk.RESPONSE_YES:
			page = self.notebook.get_current_page()
			map = self.page_map[page]
			model, rows = map["selection"].get_selected_rows()				
			iterators = [model.get_iter(row) for row in rows]
			data = [[model[row][0]] for row in rows]
			
			for iterator in iterators:
				model.remove(iterator)

			cursor = util.create_cursor()
			for table in map["delete"]:
				query = "DELETE FROM %s WHERE %s = %%s" % (table, map["id"])
				cursor.executemany(query, data)
			cursor.close()
	
	def item_edit(self, treeview, row, column):
		page = self.notebook.get_current_page()
		map = self.page_map[page]
		editor = map["editor"](map["store"][row][0], self.refresh)
		editor.dialog.show()
	
	def query(self, page, query, data=None):
		store = self.page_map[page]["store"]
		self.page_map[page]["last"] = (query, data)
		util.update_liststore(store, query, data)

	def refresh(self, *args, **kwargs):
		if "page" in kwargs:
			page = kwargs["page"]
		else:
			page = self.notebook.get_current_page()
		
		map = self.page_map[page]
		if "last" in map:
			self.query(page, *map["last"])
		else:
			self.query(page, *map["default"])
		

if __name__ == "__main__":
	winMain = WinMain()
	winMain.window.show()
	gtk.main()

