"Dialog editing movies."


#
#   MoviePaca, movie database manager.
#   Copyright (C) 2010  Joakim Soderlund
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import gtk

import util
import search


class DiaMovie():
	def __init__(self, movie_id, callback=None):
		self.builder = gtk.Builder()
		self.builder.add_from_file(os.path.join(util.DATA_PATH, "glade", "movie.glade"))
		self.builder.connect_signals(self)
		self.dialog = self.builder.get_object("dia_movie")
		
		self.movie_id = movie_id
		self.callback = callback
		
		# Basic information.
		self.lab_id = self.builder.get_object("lab_id")
		self.ent_title = self.builder.get_object("ent_title")
		self.adj_year = self.builder.get_object("adj_year")
		self.adj_duration = self.builder.get_object("adj_duration")

		cursor = util.create_cursor()
		if cursor.execute("SELECT title, year, duration FROM Movie WHERE movie_id = %s", movie_id) != 1:
			raise Exception("Invalid movie id!")
		
		title, year, duration = cursor.fetchone()
		self.lab_id.set_text(str(movie_id))
		self.ent_title.set_text(title)
		self.adj_year.set_value(year)
		self.adj_duration.set_value(duration)

		self.ls_genre_select = self.builder.get_object("ls_genre_select")
		self.cb_genre_select = self.builder.get_object("cb_genre_select")
		util.update_liststore(
			self.ls_genre_select,
			"SELECT genre_id, genre FROM Genre ORDER BY genre",
		)
		
		# Roles
		self.ls_role = self.builder.get_object("ls_role")
		self.ls_person = self.builder.get_object("ls_person")
		self.tw_person = self.builder.get_object("tw_person")
		self.roles = {}
		cursor.execute("SELECT role_id, role FROM Role")
		for role_id, role in cursor:
			self.roles[role] = role_id
			self.ls_role.append([role])

		cursor.close()
		
		# Genre
		self.ls_genre = self.builder.get_object("ls_genre")
		self.tw_genre = self.builder.get_object("tw_genre")
		self.update_genre()
		
		# MovieStaff
		self.ls_staff = self.builder.get_object("ls_staff")
		self.tw_staff = self.builder.get_object("tw_staff")
		self.update_staff()
		
		# Set multi-select
		self.tw_staff.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
		self.tw_person.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
		self.tw_genre.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
		
	
	def movie_apply(self, *args):
		query = """UPDATE Movie SET
			title = %s,
			year = %s,
			duration = %s
			WHERE movie_id = %s
		"""
		data = [
			self.ent_title.get_text(),
			str(self.adj_year.get_value()),
			str(self.adj_duration.get_value()),
			str(self.movie_id),
		]
		cursor = util.create_cursor()
		cursor.execute(query, data)
		cursor.close()
		
		if callable(self.callback):
			self.callback()
		
		self.dialog.destroy()
	
	def genre_add(self, *args):
		active = self.cb_genre_select.get_active()
		genre_id = self.ls_genre_select[active][0]
		
		cursor = util.create_cursor()
		cursor.execute("INSERT IGNORE INTO MovieGenre SET movie_id = %s, genre_id = %s", [self.movie_id, genre_id])
		cursor.close()
		self.update_genre()
		
	def genre_remove(self, *args):
		model, rows = self.tw_genre.get_selection().get_selected_rows()
		iterators = [model.get_iter(row) for row in rows]
		query = """
			DELETE FROM MovieGenre
			WHERE movie_id = %s
			AND genre_id = %s
			LIMIT 1
		"""
		data = [[self.movie_id, model[row][0]] for row in rows]
		
		for iterator in iterators:
			model.remove(iterator)

		cursor = util.create_cursor()
		cursor.executemany(query, data)
		cursor.close()
		self.update_genre()

	def update_genre(self, *args):
		cursor = util.create_cursor()
		cursor.execute("SELECT genre_id, genre FROM Genre NATURAL JOIN MovieGenre WHERE movie_id = %s", self.movie_id)
		self.ls_genre.clear()
		for genre_id, genre in cursor:
			self.ls_genre.append([genre_id, genre])
		cursor.close()
		
	def update_staff(self, *args):
		util.update_liststore(
			self.ls_staff,
			"SELECT person_id, name, role_id, role FROM MovieStaff NATURAL JOIN Person NATURAL JOIN Role WHERE movie_id = %s",
			self.movie_id,
		)
		
	
	def role_edited(self, renderer, path, role):
		if role:
			new_role_id = self.roles[role]
			old_role_id = self.ls_staff[path][2]
			person_id = self.ls_staff[path][0]
			iterator = self.ls_staff.get_iter(path)
			self.ls_staff.set_value(iterator, 3, role)
			self.ls_staff.set_value(iterator, 2, new_role_id)
			
			query = """
				UPDATE MovieStaff SET role_id = %s
				WHERE movie_id = %s
				AND person_id = %s
				AND role_id = %s
				LIMIT 1
			"""
			data = [
				new_role_id,
				self.movie_id,
				person_id,
				old_role_id,
			]
			
			cursor = util.create_cursor()
			cursor.execute(query, data)
			cursor.close()
	
	def person_search(self, *args):
		search.DialogSearch(
			search.PERSON,
			lambda query, data: util.update_liststore(self.ls_person, query, data)
		).dialog.show()

	def staff_add(self, *args):
		model, rows = self.tw_person.get_selection().get_selected_rows()
		cursor = util.create_cursor()
		query = """
			INSERT INTO MovieStaff SET
			movie_id = %s,
			person_id = %s,
			role_id = 1
		"""
		
		for row in rows:
			cursor.execute(query, (self.movie_id, model[row][0]))
		cursor.close()
		self.update_staff()
			
		
	def staff_remove(self, *args):
		model, rows = self.tw_staff.get_selection().get_selected_rows()
		cursor = util.create_cursor()
		query = """
			DELETE FROM MovieStaff
			WHERE movie_id = %s
			AND person_id = %s
			AND role_id = %s
			LIMIT 1
		"""
		
		for row in rows:
			data = model[row]
			cursor.execute(query, (self.movie_id, data[0], data[2]))
		
		cursor.close()
		self.update_staff()
