"Dialog editing persons."


#
#   MoviePaca, movie database manager.
#   Copyright (C) 2010  Joakim Soderlund
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import gtk

import util
import search

class DiaPerson():
	def __init__(self, person_id, callback=None):
		self.builder = gtk.Builder()
		self.builder.add_from_file(os.path.join(util.DATA_PATH, "glade", "person.glade"))
		self.builder.connect_signals(self)
		self.dialog = self.builder.get_object("dia_person")
		
		self.person_id = person_id
		self.callback = callback
		
		# Basic information.
		self.lab_id = self.builder.get_object("lab_id")
		self.ent_name = self.builder.get_object("ent_name")

		cursor = util.create_cursor()
		if cursor.execute("SELECT name FROM Person WHERE person_id = %s", person_id) != 1:
			raise Exception("Invalid person id!")
		
		name = cursor.fetchone()[0]
		self.lab_id.set_text(str(person_id))
		self.ent_name.set_text(name)
		
		# Roles
		self.ls_role = self.builder.get_object("ls_role")
		self.ls_movie = self.builder.get_object("ls_movie")
		self.tw_movie = self.builder.get_object("tw_movie")
		self.roles = {}
		cursor.execute("SELECT role_id, role FROM Role")
		for role_id, role in cursor:
			self.roles[role] = role_id
			self.ls_role.append([role])

		cursor.close()
		
		# MovieStaff
		self.ls_staff = self.builder.get_object("ls_staff")
		self.tw_staff = self.builder.get_object("tw_staff")
		self.update_staff()
		
		# Set multiple selection.
		self.tw_movie.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
		self.tw_staff.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
	
	def person_apply(self, *args):
		query = """UPDATE Person
			SET name = %s
			WHERE person_id = %s
		"""
		data = [
			self.ent_name.get_text(),
			str(self.person_id),
		]
		
		cursor = util.create_cursor()
		cursor.execute(query, data)
		cursor.close()
		
		if callable(self.callback):
			self.callback()
		
		self.dialog.destroy()
		
	def update_staff(self, *args):
		util.update_liststore(
			self.ls_staff,
			"SELECT movie_id, title, year, role_id, role FROM MovieStaff NATURAL JOIN Movie NATURAL JOIN Role WHERE person_id = %s",
			self.person_id,
		)
		
	def role_edited(self, renderer, path, role):
		if role:
			new_role_id = self.roles[role]
			old_role_id = self.ls_staff[path][3]
			movie_id = self.ls_staff[path][0]
			iterator = self.ls_staff.get_iter(path)
			self.ls_staff.set_value(iterator, 4, role)
			self.ls_staff.set_value(iterator, 3, new_role_id)
			
			query = """
				UPDATE MovieStaff SET role_id = %s
				WHERE movie_id = %s
				AND person_id = %s
				AND role_id = %s
				LIMIT 1
			"""
			data = [
				new_role_id,
				movie_id,
				self.person_id,
				old_role_id,
			]
			
			cursor = util.create_cursor()
			cursor.execute(query, data)
			cursor.close()
	
	def movie_search(self, *args):
		search.DialogSearch(
			search.MOVIE,
			lambda query, data: util.update_liststore(self.ls_movie, query, data)
		).dialog.show()

	def staff_add(self, *args):
		model, rows = self.tw_movie.get_selection().get_selected_rows()
		cursor = util.create_cursor()
		query = """
			INSERT INTO MovieStaff SET
			person_id = %s,
			movie_id = %s,
			role_id = 1
		"""
		
		for row in rows:
			cursor.execute(query, (self.person_id, model[row][0]))

		cursor.close()
		self.update_staff()
			
	def staff_remove(self, *args):
		model, rows = self.tw_staff.get_selection().get_selected_rows()
		cursor = util.create_cursor()
		query = """
			DELETE FROM MovieStaff
			WHERE person_id = %s
			AND movie_id = %s
			AND role_id = %s
			LIMIT 1
		"""
		
		for row in rows:
			data = model[row]
			cursor.execute(query, (self.person_id, data[0], data[3]))
		
		cursor.close()
		self.update_staff()
	
